local L = LibStub("AceLocale-3.0"):GetLocale("TrackOMatic", true);

local PREFIX = "G.TOM";

local WAIT_VERSION_REQUEST = GetTime() + 15;
local VERSION_REQUEST_SENT = false;
local VERSION_ALERT_DISPLAYED = false;
-- variables below are unused right now
local LAST_VERSION_REQUEST = {
    ["BATTLEGROUND"] = 0,
    ["RAID"] = 0,
    ["PARTY"] = 0,
};
local QUEUED_VERSION_REQUEST = {};
local GROUPED = 0;

--========================================
-- Returns whether a version request can
-- be send on the given channel
--========================================
function canSendRequest(type)
    if (type == "GUILD") then
        if (not IsInGuild()) then
            return false;
        end
    end
    return true;
end

--========================================
-- (called by TrackOMatic_OnUpdate)
-- Perform the delayed version request
-- in the guild addon channel
--========================================
function TrackOMatic_CheckVersionRequestQueue()
    if ((GetTime() > WAIT_VERSION_REQUEST) and (not VERSION_REQUEST_SENT)) then
        TrackOMatic_SendVersionRequest("GUILD");
        VERSION_REQUEST_SENT = true;
    end
end

--========================================
-- Determine what type of group the
-- player is in and queue the proper
-- version request (unused)
--========================================
function TrackOMatic_CheckGroupVersion()
    if (UnitInRaid("player")) then
        if (GROUPED ~= 2) then
            TrackOMatic_QueueVersionRequest("BATTLEGROUND");
            TrackOMatic_QueueVersionRequest("RAID");
        end
        GROUPED = 2;
    else
        if (GetNumPartyMembers() > 0) then
            if (GROUPED ~= 1) then
                TrackOMatic_QueueVersionRequest("PARTY");
            end
            GROUPED = 1;
        else
            GROUPED = 0;
        end
    end
end

--========================================
-- Queue a version request for group
-- channels (unused)
--========================================
function TrackOMatic_QueueVersionRequest(type)
    if (canSendRequest(type)) then
        if ((GetTime() - LAST_VERSION_REQUEST[type]) >= 10) then
            TrackOMatic_SendVersionRequest(type);
            LAST_VERSION_REQUEST[type] = GetTime();
        else
            QUEUED_VERSION_REQUEST[type] = true;
        end
    end
end

--========================================
-- Register addon message prefix
--========================================
function TrackOMatic_RegisterMessagePrefix()
    RegisterAddonMessagePrefix(PREFIX);
end

--========================================
-- Addon message handler
--========================================
function TrackOMatic_OnAddonMessage(prefix, message, type, sender)
    if (prefix == "G.TOM") then
        local mc, arg1, arg2 = strsplit("|", message);
        if ((mc == "VC") or (mc == "VR")) then
            local vid, ver = (tonumber(arg1) or 0), arg2;
            if (vid > TRACKOMATIC_VERSIONID) then
                if (vid > TRACKOMATIC_HIGHEST_VERSION_SEEN.number) then
                    TRACKOMATIC_HIGHEST_VERSION_SEEN = {number = vid, version = ver};
                end
                if (not VERSION_ALERT_DISPLAYED) then
                    TrackOMatic_Message(string.format(L["NEWER_VERSION"], ver));
                    VERSION_ALERT_DISPLAYED = true;
                end
            end
            -- only send a response (VR) message on a check (VC), otherwise everyone's addons will be constantly bouncing responses at each other...
            if (mc == "VC") then
                TrackOMatic_SendVersionResponse("WHISPER", sender);
            end
        end
    end
end

--========================================
-- Sends version REQUEST message
--========================================
function TrackOMatic_SendVersionRequest(type, target)
    if ((type == "GUILD") and (not IsInGuild())) then return false; end
    SendAddonMessage(PREFIX, "VC|" .. TRACKOMATIC_VERSIONID .. "|" .. TRACKOMATIC_VERSION, type, target);
    return true;
end

--========================================
-- Sends version RESPONSE message
--========================================
function TrackOMatic_SendVersionResponse(type, target)
    SendAddonMessage("G.TOM", "VR|" .. TRACKOMATIC_VERSIONID .. "|" .. TRACKOMATIC_VERSION, type, target);
end
